% Filter for indexing a data container with filter for rows / cols
% not yet any other supported filtering or method
% this is useful to avoid copying large memory blocks (applying the filter)
% or if such copying is not possible - e.g. Datacontainer is approximated
% behave to some degree like a regular matrix
% Input (squared) proximity matrix
% N - number of points accessible in Data
% - not yet with real interfaces the OO in matlab is a bit fresh
classdef DataContainerFilter  < matlab.mixin.Copyable
    properties (GetAccess='private',SetAccess = 'private')
        RowFilter       = [];
        ColFilter       = [];    
        m_N             = 0;
        m_M             = 0;
        DataContainer   = [];
    end
    
    methods
        % constructor
        function obj = DataContainerFilter(oDataContainer,RowFilter,ColFilter)
            if(iscell(oDataContainer))
                RowFilter = oDataContainer{1};
                ColFilter = oDataContainer{2};
                ClassName = oDataContainer{3};
                if(~ischar(ClassName))
                    oDataContainer = ClassName;
                else
                    oDataContainer = feval(ClassName,oDataContainer{4:end});
                end
            end
            assert(isa(oDataContainer, 'DataContainer') ||isa(oDataContainer, 'DataContainerNormalizer') ||isa(oDataContainer, 'DataContainerFilter'))
            obj.DataContainer = oDataContainer;
            [N,M] = size(oDataContainer);
            if(~isnumeric(ColFilter) && strcmp(ColFilter,':'))
                ColFilter = 1:M;
            end
            if(~isnumeric(RowFilter) && strcmp(RowFilter,':'))
                RowFilter = 1:N;
            end
            assert(numel(unique(RowFilter))<=N);
            assert(numel(unique(ColFilter))<=M);
            obj.RowFilter       = RowFilter;            
            obj.ColFilter       = ColFilter;            
            obj.m_N             = numel(RowFilter);
            obj.m_M             = numel(ColFilter);
        end          

        % returns the parameter (not data matrix) of the binded container
        function oCell = GetParameter(me)
            oCell = {me.RowFilter,me.ColFilter,class(me.DataContainer),me.DataContainer.GetParameter()};
        end
        
        % input 
        % X - index vector 
        % Y - index vector 
        function ret = GetData(me,X,Y)
            if(~exist('X','var'))       
                ret = me.DataContainer.GetData(me.RowFilter,me.ColFilter);
            else
                X=X(:);
                Y=Y(:);
                ret = me.DataContainer.GetData(me.RowFilter(X),me.ColFilter(Y));
            end
        end
                
        function [N,M] = size(me,varargin)
            N = [];
            M = [];
            if(nargin>1)
                switch varargin{1}
                    case 1,
                        N = me.m_N;
                    case 2
                        N = me.m_M;
                end
            else
                N = me.m_N;
                M = me.m_M;
            end
        end

        
        function Value = subsref(me,Indices)
            Value = [];
            NumInd = numel(Indices);
            if(NumInd==1) % direct access 
                Subs       = Indices.subs;
                NumIndSubs = numel(Subs);
                switch Indices.type
                    case '()'     % indexing                        
                        if(NumIndSubs==2)
                            Value = GetData(me,(Subs{1}),(Subs{2}));
                        else
                            ME = MException('Class:DataContainer:Error','number of parameters must be two');
                            throw(ME);
                        end
                    case '.'     % class member
                        Value = me.(Subs);
                end
            else % should be a function call
                if(strncmp(Indices(1).type,'.',1))   
                    Value = builtin('subsref', me, Indices); 
                end
            end
        end       
    end
    
     methods(Access = protected)
       % Override copyElement method:
      function cpObj = copyElement(obj)
         % Make a shallow copy of all four properties
         cpObj = copyElement@matlab.mixin.Copyable(obj);      
      end
     end
end